## Script (Python) "skin_properties"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
prop_dict = [ {"id":"body_font",		"type":"string"},
              {"id":"background_color",	"type":"string"},
              {"id":"font_color",		"type":"string"},
              {"id":"banner_font",		"type":"string"},
              {"id":"banner_color",		"type":"string"},
              {"id":"color1",			"type":"string"},
              {"id":"color2",			"type":"string"},
              {"id":"color3",			"type":"string"},
              {"id":"color4",			"type":"string"},
              {"id":"sidebox_background","type":"string"}
            ]

container.update_footer_prop()

return prop_dict
