##############################################################################
#
# permissions.py
# Permission setting name for COREBlog
#
# Copyright (c) 2003-2005 Atsushi Shibata. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
##############################################################################

#Permission settings

__doc__="""Zope Blog Product 'COREBlog:permissions'
$Id: permissions.py,v 1.2 2005/01/28 08:37:02 ats_shib Exp $"""

__version__='$Revision: 1.2 $'[11:-2]

View = "View"

ManageCOREBlog = "Manage COREBlog"
AddCOREBlogEntries = "Add COREBlog Entries"
AddCOREBlogComments = "Add COREBlog Comments"
ModerateCOREBlogEntries = "Moderate COREBlog Entries"
