## Script (Python) "skin_objects"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
#plonified 0.95 - the Plonified COREBlog skin
#   (c) 2005 A.Shibata

ov = context.objectValues()
ol = []
for v in ov:
    try:
        ol.append(v.id())
    except:
        ol.append(v.id)

ol.remove('skin_objects')
ol.remove('skin_properties')
return ol
