## Script (Python) "getIcon"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
return 'misc_/COREBlog/coreblog_img'
