object_list = ["blog_banner","blog_footer","blog_header", \
               "categorylist_html","comment_body","comment_form", \
               "daylist_html","entry_body","entry_html", \
               "entry_rdf","index_html","monthlist_html", \
               "previewcomment_html","style_css","trackback_body", \
               "search_results", \
               "skin_setting_form"
               ]

return object_list
